@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2023 A S Lewis
+# Copyright (C) 2019-2025 A S Lewis
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -33,7 +33,7 @@ import re
 import formats
 import mainapp
 import media
-import utils
+import ttutils
 # Use same gettext translations
 from mainapp import _
 
@@ -541,6 +541,18 @@ class OptionsManager(object):
             the source URL of each affected media data object(s) is used, as
             normal. Ignored if 'direct_cmd_flag' is False
 
+        [ fetch options ]
+
+        fetch_formats_cmd_string (str): String that contains extra youtube-dl
+            options separated by spaces, used only during an info operation to
+            fetch a list of formats for a video. Components containing
+            whitespace can be enclosed within double quotes "..."
+
+        fetch_subtitles_cmd_string (str): String that contains extra youtube-dl
+            options separated by spaces, used only during an info operation to
+            fetch a list of subtitles for a video. Components containing
+            whitespace can be enclosed within double quotes "..."
+
     TARTUBE OPTIONS (not passed to youtube-dl directly)
 
         move_description (bool):
@@ -615,9 +627,9 @@ class OptionsManager(object):
         video_format_mode (str): 'all' to download all available formats,
             ignoring the preference list (sets the option 'all_formats').
             'single' to download the first available format in
-            'video_format_list'. 'single_agree' to download the first format in
-            'video_format_list' that's available for all videos. 'multiple' to
-            download all available formats in 'video_format_list'
+            'video_format_list'. 'combine' to download the first video and the
+            first audio format in the list, to be combined as a single file,
+            'multiple' to download all available formats in 'video_format_list'
 
         subs_lang_list (list): List of language tags which are used to set
             the 'subs_lang' option
@@ -725,10 +737,11 @@ class OptionsManager(object):
 
             if code != '0':
 
-                if formats.VIDEO_OPTION_TYPE_DICT[code] is False:
-                    video_list.append(code)
-                else:
+                if code in formats.VIDEO_OPTION_TYPE_DICT and \
+                formats.VIDEO_OPTION_TYPE_DICT[code] == 'audio':
                     audio_list.append(code)
+                else:
+                    video_list.append(code)
 
         comb_list.extend(video_list)
         comb_list.extend(audio_list)
@@ -900,6 +913,8 @@ class OptionsManager(object):
             'extra_cmd_string': '',
             'direct_cmd_flag': False,
             'direct_url_flag': False,
+            'fetch_formats_cmd_string': '',
+            'fetch_subtitles_cmd_string': '',
             # TARTUBE OPTIONS
             'move_description': False,
             'move_info': False,
@@ -917,7 +932,7 @@ class OptionsManager(object):
             'match_title_list': [],
             'reject_title_list': [],
             'video_format_list': [],
-            'video_format_mode': 'single',
+            'video_format_mode': 'combine',
             'subs_lang_list': [ 'en' ],
             'downloader_config': False,
         }
@@ -1377,6 +1392,8 @@ class OptionsParser(object):
 #           OptionHolder('extra_cmd_string', '', ''),
 #           OptionHolder('direct_cmd_flag', '', False),
 #           OptionHolder('direct_url_flag', '', False),
+#           OptionHolder('fetch_formats_cmd_string', '', ''),
+#           OptionHolder('fetch_subtitles_cmd_string', '', ''),
             # TARTUBE OPTIONS (not given an options.OptionHolder object)
 #           OptionHolder('move_description', '', False),
 #           OptionHolder('move_info', '', False),
@@ -1394,7 +1411,7 @@ class OptionsParser(object):
 #           OptionHolder('match_title_list', '', []),
 #           OptionHolder('reject_title_list', '', []),
 #           OptionHolder('video_format_list', '', []),
-#           OptionHolder('video_format_mode', '', 'single'),
+#           OptionHolder('video_format_mode', '', 'combine'),
 #           OptionHolder('subs_lang_list', '', []),
 #           OptionHolder('downloader_config', '', False),
         ]
@@ -1476,7 +1493,7 @@ class OptionsParser(object):
                 and value != option_holder_obj.default_value:
                     options_list.append('-x')
                     options_list.append(option_holder_obj.switch)
-                    options_list.append(utils.to_string(value))
+                    options_list.append(ttutils.to_string(value))
 
                     # The '-x' / '--audio-quality' switch must precede the
                     #   '--audio-quality' switch, if both are used
@@ -1487,7 +1504,7 @@ class OptionsParser(object):
                     if copy_dict['audio_quality'] != '5':
                         options_list.append('--audio-quality')
                         options_list.append(
-                            utils.to_string(copy_dict['audio_quality']),
+                            ttutils.to_string(copy_dict['audio_quality']),
                         )
 
             elif option_holder_obj.name == 'audio_quality':
@@ -1499,16 +1516,16 @@ class OptionsParser(object):
 
                         if value != option_holder_obj.default_value:
                             options_list.append(option_holder_obj.switch)
-                            options_list.append(utils.to_string(value))
+                            options_list.append(ttutils.to_string(value))
 
             elif option_holder_obj.name == 'match_filter':
-                value = utils.to_string(copy_dict[option_holder_obj.name])
+                value = ttutils.to_string(copy_dict[option_holder_obj.name])
                 if self.app_obj.block_livestreams_flag:
 
                     if value == '':
                         value = '!is_live'
                     else:
-                        value += ' \& !is_live'
+                        value += r' & !is_live'
 
                 if value != '':
                     options_list.append(option_holder_obj.switch)
@@ -1519,7 +1536,7 @@ class OptionsParser(object):
                 value = copy_dict[option_holder_obj.name]
                 if value != '':
                     options_list.append(option_holder_obj.switch)
-                    options_list.append('"' + utils.to_string(value) + '"')
+                    options_list.append('"' + ttutils.to_string(value) + '"')
 
             elif option_holder_obj.name == 'cookies_path':
                 cookies_path = copy_dict[option_holder_obj.name]
@@ -1557,7 +1574,7 @@ class OptionsParser(object):
                     options_list.append(option_holder_obj.switch)
 
                     if not option_holder_obj.is_boolean():
-                        options_list.append(utils.to_string(value))
+                        options_list.append(ttutils.to_string(value))
 
         # Parse the 'match_title_list' and 'reject_title_list'
         for item in copy_dict['match_title_list']:
@@ -1582,8 +1599,18 @@ class OptionsParser(object):
             options_list.append('--extractor-args')
             options_list.append(item)
 
+        # Add the SponsorBlock option used in Classic Mode downloads
+        if (
+            operation_type == 'classic_sim' \
+            or operation_type == 'classic_real' \
+            or operation_type == 'classic_custom'
+        ) and media_data_obj.dummy_sblock_flag:
+            options_list.append('--sponsorblock-remove')
+            options_list.append('default')
+
         # Build the --output and --paths options
         options_list = self.build_paths(
+            media_data_obj,
             dir_path,
             copy_dict,
             options_list,
@@ -1592,7 +1619,7 @@ class OptionsParser(object):
         # Parse the 'extra_cmd_string' option, so it overrules everything else.
         #   The option can contain arguments inside double quotes "..."
         #   (arguments that can therefore contain whitespace)
-        parsed_list = utils.parse_options(copy_dict['extra_cmd_string'])
+        parsed_list = ttutils.parse_options(copy_dict['extra_cmd_string'])
         for item in parsed_list:
             options_list.append(item)
 
@@ -1601,7 +1628,7 @@ class OptionsParser(object):
         if copy_dict['downloader_config']:
 
             options_list.append('--config-location')
-            options_list.append(utils.get_dl_config_path(self.app_obj))
+            options_list.append(ttutils.get_dl_config_path(self.app_obj))
 
         # Filter out yt-dlp options, if required. A list of them is specified
         #   in mainapp.TartubeApp.ytdlp_exclusive_options_dict
@@ -1644,12 +1671,12 @@ class OptionsParser(object):
 
         if copy_dict['min_filesize']:
             copy_dict['min_filesize'] = \
-            utils.to_string(copy_dict['min_filesize']) + \
+            ttutils.to_string(copy_dict['min_filesize']) + \
             copy_dict['min_filesize_unit']
 
         if copy_dict['max_filesize']:
             copy_dict['max_filesize'] = \
-            utils.to_string(copy_dict['max_filesize']) + \
+            ttutils.to_string(copy_dict['max_filesize']) + \
             copy_dict['max_filesize_unit']
 
 
@@ -1780,7 +1807,7 @@ class OptionsParser(object):
         return dir_path
 
 
-    def build_paths(self, dir_path, copy_dict, options_list):
+    def build_paths(self, media_data_obj, dir_path, copy_dict, options_list):
 
         """Called by self.parse().
 
@@ -1789,6 +1816,9 @@ class OptionsParser(object):
 
         Args:
 
+            media_data_obj (media.Video, media.Channel, media.Playlist,
+                media.Folder): The media data object being downloaded
+
             dir_path (str): The directory into which files are to be downloaded
 
             copy_dict (dict): Copy of the original options dictionary
@@ -1840,6 +1870,19 @@ class OptionsParser(object):
                 options_list.append('--paths')
                 options_list.append(item)
 
+        # Lastly, apply the output override for a media.Video object, if one
+        #   is specified
+        if media_data_obj.dbid in self.app_obj.temp_output_override_dict:
+
+            # Add it to the end of 'options_list', so anyone examining the
+            #   yellow text in the Output Tab can clearly see that this is
+            #   an override
+            name = self.app_obj.temp_output_override_dict[media_data_obj.dbid]
+            options_list.append('--output')
+            options_list.append(
+                os.path.abspath(os.path.join(dir_path, name + '.%(ext)s')),
+            )
+
         return options_list
 
 
@@ -1882,74 +1925,114 @@ class OptionsParser(object):
                 # Valid values are those specified by formats.VIDEO_FORMAT_LIST
                 #   formats.AUDIO_FORMAT_LIST and formats.VIDEO_RESOLUTION_LIST
                 convert_flag, this_format, this_res \
-                = utils.extract_dummy_format(media_data_obj.dummy_format)
+                = ttutils.extract_dummy_format(media_data_obj.dummy_format)
 
-                if not convert_flag and this_res is None:
+                if not convert_flag:
 
                     # Download the video in the specified format and
                     #   resolution, if available
 
-                    # Ignore all video/audio formats except the one specified
-                    #   by the user in the Classic Mode tab
-                    copy_dict['video_format'] = this_format
-                    copy_dict['all_formats'] = False
-                    copy_dict['video_format_list'] = []
-                    copy_dict['video_format_mode'] = ''
-                    copy_dict['recode_video'] = ''
+                    # e.g. Default > 720p > Download in this format
+                    if this_format is None and this_res is not None:
 
-                    # v2.1.009: Since the user doesn't have the possibility of
-                    #   setting the -f and --merge-output-format options to the
-                    #   same value (e.g. 'mp4'), we must do so artificially
-                    # yt-dlp supports: avi, flv, mkv, mov, mp4, webm
-                    if this_format in formats.VIDEO_FORMAT_DICT \
-                    and this_format != 'ogg':
-                        copy_dict['merge_output_format'] = this_format
-
-                    return
-
-                elif this_format is not None \
-                and this_format in formats.VIDEO_FORMAT_DICT:
-
-                    # Converting video formats requires post-processing
-                    # Ignore all video/audio formats except the one specified
-                    #   by the user in the Classic Mode tab
-                    if this_res is None:
-                        copy_dict['video_format'] = '0'
-                    else:
-                        copy_dict['video_format'] \
-                        = 'bestvideo[height<=?' + this_res + ']' \
+                        copy_dict['video_format'] = \
+                        'bestvideo[height<=?' + this_res + ']' \
                         + '+bestaudio/best[height<=?' + this_res + ']'
+                        copy_dict['all_formats'] = False
+                        copy_dict['video_format_list'] = []
+                        copy_dict['video_format_mode'] = ''
+                        copy_dict['recode_video'] = ''
 
-                    copy_dict['all_formats'] = False
-                    copy_dict['video_format_list'] = []
-                    copy_dict['video_format_mode'] = ''
-                    copy_dict['recode_video'] = this_format
+                        return
 
-                    return
+                    # e.g. mp4 > Highest > Download in this format
+                    elif this_format is not None and this_res is None:
 
-                elif this_format is not None \
-                and this_format in formats.AUDIO_FORMAT_DICT:
+                        copy_dict['video_format'] = this_format
+                        copy_dict['all_formats'] = False
+                        copy_dict['video_format_list'] = []
+                        copy_dict['video_format_mode'] = ''
+                        copy_dict['recode_video'] = ''
 
-                    # Converting audio formats requires post-processing
-                    copy_dict['video_format'] = '0'
-                    copy_dict['all_formats'] = False
-                    copy_dict['video_format_list'] = []
-                    copy_dict['video_format_mode'] = ''
-                    copy_dict['extract_audio'] = True
-                    copy_dict['audio_format'] = this_format
-                    copy_dict['recode_video'] = ''
+                        # v2.1.009: Since the user doesn't have the possibility
+                        #   of setting the -f and --merge-output-format options
+                        #   to the same value (e.g. 'mp4'), we must do so
+                        #   artificially
+                        # yt-dlp supports: avi, flv, mkv, mov, mp4, webm
+                        if this_format in formats.VIDEO_FORMAT_DICT \
+                        and this_format != 'ogg':
+                            copy_dict['merge_output_format'] = this_format
 
-                    return
+                        return
 
-                elif this_format is None and this_res is not None:
+                    # e.g. mp4 > 720p > Download in this format
+                    else:
 
-                    copy_dict['video_format'] = \
-                    'bestvideo[height<=?' + this_res + ']' \
-                    + '+bestaudio/best[height<=?' + this_res + ']'
-                    copy_dict['all_formats'] = False
-                    copy_dict['video_format_list'] = []
-                    copy_dict['video_format_mode'] = ''
-                    copy_dict['recode_video'] = ''
+                        copy_dict['video_format'] = \
+                        'bestvideo[height<=?' + this_res + ']' \
+                        + '+bestaudio/best[height<=?' + this_res + ']' \
+                        + '[ext=' + this_format + ']'
+                        copy_dict['all_formats'] = False
+                        copy_dict['video_format_list'] = []
+                        copy_dict['video_format_mode'] = ''
+                        copy_dict['recode_video'] = ''
+
+                        if this_format in formats.VIDEO_FORMAT_DICT \
+                        and this_format != 'ogg':
+                            copy_dict['merge_output_format'] = this_format
+
+                        return
+
+                else:
+
+                    # Convert the video after download (which requires
+                    #   post-processing)
+
+                    # e.g. Default > 720p > Convert to this format
+                    # N.B. This cannot actually be selected
+#                   if this_format is None and this_res is not None:
+#
+#                       copy_dict['video_format'] = \
+#                       'bestvideo[height<=?' + this_res + ']' \
+#                       + '+bestaudio/best[height<=?' + this_res + ']'
+#                       copy_dict['all_formats'] = False
+#                       copy_dict['video_format_list'] = []
+#                       copy_dict['video_format_mode'] = ''
+#                       copy_dict['recode_video'] = ''
+#
+#                       return
+
+                    # e.g. mp4 > Highest > Convert to this format
+                    # e.g. mp4 > 720p > Convert to this format
+                    if this_format is not None \
+                    and this_format in formats.VIDEO_FORMAT_DICT:
+
+                        if this_res is None:
+                            copy_dict['video_format'] = '0'
+                        else:
+                            copy_dict['video_format'] \
+                            = 'bestvideo[height<=?' + this_res + ']' \
+                            + '+bestaudio/best[height<=?' + this_res + ']'
+
+                        copy_dict['all_formats'] = False
+                        copy_dict['video_format_list'] = []
+                        copy_dict['video_format_mode'] = ''
+                        copy_dict['recode_video'] = this_format
+
+                        return
+
+                    elif this_format is not None \
+                    and this_format in formats.AUDIO_FORMAT_DICT:
+
+                        copy_dict['video_format'] = '0'
+                        copy_dict['all_formats'] = False
+                        copy_dict['video_format_list'] = []
+                        copy_dict['video_format_mode'] = ''
+                        copy_dict['extract_audio'] = True
+                        copy_dict['audio_format'] = this_format
+                        copy_dict['recode_video'] = ''
+
+                        return
 
         # Special case: for simulated downloads, don't specify any video
         #   formats; if the format isn't available for some videos, we'll get
@@ -2030,19 +2113,50 @@ class OptionsParser(object):
                 copy_dict['video_format'] = 0
                 copy_dict['all_formats'] = True
 
-            else:
-
+            elif video_format_mode == 'multiple':
+                copy_dict['video_format'] = ','.join(video_format_list)
                 copy_dict['all_formats'] = False
 
-                if video_format_mode == 'single_agree':
-                    char = '/'
-                elif video_format_mode == 'multiple':
-                    char = ','
-                else:
-                    # mode is 'single'
-                    char = '+'
+            elif video_format_mode == 'combine':
 
-                copy_dict['video_format'] = char.join(video_format_list)
+                # If 'video_format_list' consists only of video-only and
+                #   audio-only formats, then they can be combined in the form
+                #       -f (V/V/V...)+(A/A/A...)
+                # Otherwise, treat 'combine' the same as 'single', combining
+                #   the formats in the form
+                #       -f V/V/V/A/A/A...
+                # (See Git #680)
+
+                video_only_list = []
+                audio_only_list = []
+                default_list = []
+                for value in video_format_list:
+                    if not value in formats.VIDEO_OPTION_TYPE_DICT:
+                        default_list.append(value)
+                    elif formats.VIDEO_OPTION_TYPE_DICT[value] == 'video':
+                        video_only_list.append(value)
+                    elif formats.VIDEO_OPTION_TYPE_DICT[value] == 'audio':
+                        audio_only_list.append(value)
+                    else:
+                        default_list.append(value)
+
+                if not default_list and video_only_list and audio_only_list:
+                    copy_dict['video_format'] = '(' + \
+                    '/'.join(video_only_list) \
+                    + ')+(' \
+                    + '/'.join(audio_only_list) \
+                    + ')'
+                    copy_dict['all_formats'] = False
+
+                else:
+                    # Handle 'video_format_mode' as if its value were 'single'
+                    copy_dict['video_format'] = '/'.join(video_format_list)
+                    copy_dict['all_formats'] = False
+
+            else:
+                # 'video_format_mode' is 'single'
+                copy_dict['video_format'] = '/'.join(video_format_list)
+                copy_dict['all_formats'] = False
 
             copy_dict['video_format_list'] = []
             copy_dict['video_format_mode'] = ''
