@@ -1,20 +1,20 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2022 A S Lewis
+# Copyright (C) 2019-2025 A S Lewis
 #
-# This program is free software: you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any later
-# version.
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation; either version 2.1 of the License, or (at your option)
+# any later version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 # details.
 #
-# You should have received a copy of the GNU General Public License along with
-# this program. If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 
 """Refresh operation classes."""
@@ -34,7 +34,6 @@ import time
 # Import our modules
 import formats
 import media
-import utils
 # Use same gettext translations
 from mainapp import _
 
@@ -159,7 +158,7 @@ class RefreshManager(threading.Thread):
             obj_list = self.init_obj.compile_all_containers(obj_list)
         else:
             # Add all channels/playlists/folders in the database
-            for dbid in list(self.app_obj.media_name_dict.values()):
+            for dbid in self.app_obj.container_reg_dict.keys():
 
                 obj = self.app_obj.media_reg_dict[dbid]
                 # Don't add private folders
@@ -286,13 +285,20 @@ class RefreshManager(threading.Thread):
             if not self.running_flag:
                 return
 
-            filename, ext = os.path.splitext(relative_path)
-            # (Remove the initial .)
-            ext = ext[1:]
-            if ext in formats.VIDEO_FORMAT_DICT \
-            or ext in formats.AUDIO_FORMAT_DICT:
+            # (Don't handle unwisely-named directories...)
+            if os.path.isfile(
+                os.path.abspath(
+                    os.path.join(dir_path, relative_path),
+                ),
+            ):
 
-                mod_list.append(relative_path)
+                filename, ext = os.path.splitext(relative_path)
+                # (Remove the initial .)
+                ext = ext[1:]
+                if ext in formats.VIDEO_FORMAT_DICT \
+                or ext in formats.AUDIO_FORMAT_DICT:
+
+                    mod_list.append(relative_path)
 
         # From the new list, filter out duplicate filenames (e.g. if the list
         #   contains both 'my_video.mp4' and 'my_video.webm', filter out the
@@ -399,6 +405,13 @@ class RefreshManager(threading.Thread):
                     if not child_relative_path in alt_list:
                         child_obj.set_file(filename, ext)
 
+                # Take this opportunity to update the video file size (etc),
+                #   in case they weren't set during the original download
+                self.app_obj.update_video_from_filesystem(
+                    child_obj,
+                    child_obj.get_actual_path(self.app_obj),
+                )
+
                 # Eliminate this media.Video object; no other video file should
                 #   match it
                 del check_dict[filename]
