@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2019-2023 A S Lewis
+# Copyright (C) 2019-2025 A S Lewis
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU Lesser General Public License as published by the Free
@@ -41,7 +41,7 @@ import time
 # Import our modules
 import formats
 import media
-import utils
+import ttutils
 # Use same gettext translations
 from mainapp import _
 
@@ -989,7 +989,7 @@ class TidyManager(threading.Thread):
             check_list.extend(file_name_list)
 
         char = '|'
-        regex = '\.f\d+\.(' + char.join(formats.VIDEO_FORMAT_LIST) + ')$'
+        regex = r'\.f\d+\.(' + char.join(formats.VIDEO_FORMAT_LIST) + ')$'
         for check_path in check_list:
             if re.search(regex, check_path):
 
@@ -1126,7 +1126,7 @@ class TidyManager(threading.Thread):
                 #   thumbnail has already been moved into /.thumbs, then of
                 #   course we don't move it again (and this function returns an
                 #   empty list)
-                path_list = utils.find_thumbnail_restricted(
+                path_list = ttutils.find_thumbnail_restricted(
                     self.app_obj,
                     video_obj,
                 )
@@ -1185,7 +1185,7 @@ class TidyManager(threading.Thread):
             if video_obj.file_name is not None:
 
                 # Thumbnails might be in one of four locations
-                thumb_path = utils.find_thumbnail(self.app_obj, video_obj)
+                thumb_path = ttutils.find_thumbnail(self.app_obj, video_obj)
 
                 # If the video's parent container has an alternative download
                 #   destination set, we must check the corresponding media
@@ -1226,7 +1226,7 @@ class TidyManager(threading.Thread):
             if video_obj.file_name is not None:
 
                 # Thumbnails might be in one of two locations
-                webp_path = utils.find_thumbnail_webp_strict(
+                webp_path = ttutils.find_thumbnail_webp_strict(
                     self.app_obj,
                     video_obj
                 )
@@ -1271,7 +1271,7 @@ class TidyManager(threading.Thread):
             if video_obj.file_name is not None:
 
                 # Thumbnails might be in one of four locations
-                thumb_path = utils.find_thumbnail_webp_intact_or_broken(
+                thumb_path = ttutils.find_thumbnail_webp_intact_or_broken(
                     self.app_obj,
                     video_obj,
                 )
